import {
  Body,
  Controller,
  Get,
  NotAcceptableException,
  Param,
  Post,
  Query,
  Res,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { Response } from 'express';
import { Account } from 'src/account/entities/account.entity';
import { CurrentUser } from 'src/auth/decorators/current-user.decorator';
import { Roles } from 'src/auth/decorators/roles.decorator';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { PaymentStatus, PaymentType, UserRole } from 'src/enum';
import { NotifyService } from 'src/notify/notify.service';
import { checkPaymentStatus, generateUrl } from 'src/utils/payment.util';
import { instance } from 'src/utils/razor-pay.utils';
import {
  OrderDto,
  PaginationDto,
  PayDto,
  PaymentHistoryDto,
  verifyPaymentDto,
} from './dto/payment-history.dto';
import { PaymentHistoryService } from './payment-history.service';
import { CommonPaginationDto } from 'src/common/dto/common-pagination.dto';
import { createInvoice } from 'src/utils/pdf-invoice.utils';

@Controller('payment-history')
export class PaymentHistoryController {
  constructor(
    private readonly paymentHistoryService: PaymentHistoryService,
    private readonly notifyService: NotifyService,
  ) {}

  @Post('order')
  // @UseGuards(AuthGuard('jwt'))
  async createOrder(@Body() dto: OrderDto) {
    return await this.paymentHistoryService.createOrder(dto);
  }

  @Post('save-payment')
  @UseGuards(AuthGuard('jwt'))
  async savePayment(
    @Body() dto: verifyPaymentDto,
    @CurrentUser() user: Account,
  ) {
    return this.paymentHistoryService.savePayment(dto, user.id);
  }

  @Get('all/list')
  @UseGuards(AuthGuard('jwt'), RolesGuard)
  @Roles(UserRole.BUSINESS)
  findAll(@Query() dto: PaginationDto, @CurrentUser() user: Account) {
    return this.paymentHistoryService.find(dto, user.id);
  }

  @Post('pay')
  async payDoctor(@Body() dto, @Res() res: Response) {
    const result = await checkPaymentStatus(dto.transactionId);
    if (!result.success) {
      return res.redirect(
        process.env.STN_REDIRECT_URL + '/failed/' + dto.transactionId,
      );
    } else {
      return res.redirect(
        process.env.STN_REDIRECT_URL + '/success/' + dto.transactionId,
      );
    }
  }

  @Get('admin/list/total')
  @UseGuards(AuthGuard('jwt'), RolesGuard)
  @Roles(UserRole.ADMIN)
  findAllByAdminTotal(@Query() dto: PaginationDto) {
    return this.paymentHistoryService.findTotal(dto);
  }

  @Get('member/payment-list')
  @UseGuards(AuthGuard('jwt'), RolesGuard)
  @Roles(UserRole.USER)
  paymentList(@Query() dto: CommonPaginationDto, @CurrentUser() user: Account) {
    return this.paymentHistoryService.paymentList(dto, user.id);
  }

  @Get('payment-detail/member/:id')
  @UseGuards(AuthGuard('jwt'), RolesGuard)
  @Roles(UserRole.USER)
  async getPaymentDetail(@Param('id') id: string, @Res() res: Response) {
    const paymentDetail = await this.paymentHistoryService.getPaymentDetail(id);
    const invoice = {
      transactionId: paymentDetail.transactionId,
      invoiceNumber: paymentDetail.invoiceNumber,
      paymentDate: new Date(paymentDetail.createdAt).toLocaleDateString(),
      mode: 'Online',
      amount: paymentDetail.amount,
      total: paymentDetail.total,
      membershipCardName: paymentDetail.userDetail['membershipCard']['name'],
      duration: paymentDetail.userDetail['membershipCard']['validity'],
      userDetail: {
        name:
          paymentDetail.userDetail['fName'] + ' ' +
          `${paymentDetail.userDetail['mName'] || null}` + ' ' +
          paymentDetail.userDetail['lName'],
        memberId: paymentDetail.userDetail['memberId'],
        address: paymentDetail.userDetail['address1'],
        from: paymentDetail.userDetail['membershipValidFrom'],
        to: paymentDetail.userDetail['membershipValidTo'],
      },
      business: {
        businessName: paymentDetail.account['business']['0']['businessName'],
        gstNo: paymentDetail.account['business']['0']['gstNo'],
        address: paymentDetail.account['business']['0']['address1'],
      },
      tax: paymentDetail.account['tax'],
    };

    const pdf = await createInvoice(invoice);
    const name = Date.now().toString() + `-invoice_${paymentDetail.userDetail['memberId']}.pdf`;
    res.setHeader('Content-Type', 'application/pdf');
    res.set('Content-Disposition', `attachment; filename="${name}"`);
    pdf.pipe(res);
    pdf.end();
  }
}
